"""Inspection pass types."""


class PassType:
    """Codes for PIGPASS."""

    COMPLEX = '2088658'
    COMPLEX_SKS = '2088661'
    COMPLEX_NAV = '5345726'
    NAVIGATE = '990003167701'
    CALIPER = '2088660'
    TFI = '2088662'
    ULTRASONIC = '2088663'
    EXPERIMENTAL = '2088664'
    EMAP = '6943277'
